# Generated by Django 3.2.13 on 2022-09-07 02:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0008_rename_type_entrytype_entry_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='communicationslogentry',
            name='customer',
            field=models.IntegerField(null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-07 00:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0007_alter_entrytype_options'),
    ]

    operations = [
        migrations.RenameField(
            model_name='entrytype',
            old_name='type',
            new_name='entry_type',
        ),
    ]



from django.contrib.contenttypes.models import ContentType
from rest_framework import serializers

from org_model_logs.models import CommunicationsLogEntry, EntryType, UserAction


class EntryTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = EntryType
        fields = "__all__"


class UserActionSerializer(serializers.ModelSerializer):
    # In cases where we want to attach documents to a user action it makes it a lot easier
    # to have the content
    user_action_content_type_id = serializers.SerializerMethodField(
        required=False, allow_null=True
    )

    class Meta:
        model = UserAction
        fields = [
            "id",
            "user_action_content_type_id",
            "object_id",
            "content_type",
            "who",
            "when",
            "what",
            "why",
            "documents",
        ]
        datatables_always_serialize = [
            "id",
            "who",
            "when",
            "what",
            "documents",
        ]

    def get_user_action_content_type_id(self, obj):
        return ContentType.objects.get_for_model(UserAction).id


class CommunicationsLogEntrySerializer(serializers.ModelSerializer):
    entry_type_display_name = serializers.CharField(source="entry_type.entry_type")

    class Meta:
        model = CommunicationsLogEntry
        fields = "__all__"


class CreateCommunicationsLogEntrySerializer(serializers.ModelSerializer):
    app_label = serializers.CharField(required=False)
    model = serializers.CharField(required=False)

    class Meta:
        model = CommunicationsLogEntry
        fields = "__all__"
        # We populate these in the perform_create method of the viewset
        extra_kwargs = {
            "staff": {"required": False},
            "content_type": {"required": False},
        }



# Generated by Django 3.2.13 on 2022-09-02 07:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0006_auto_20220902_1357'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='entrytype',
            options={'ordering': ['id'], 'verbose_name': 'Entry Type', 'verbose_name_plural': 'Entry Types'},
        ),
    ]



from django.contrib import admin

from org_model_logs.models import CommunicationsLogEntry, EntryType, UserAction


class EntryTypeAdmin(admin.ModelAdmin):
    model = EntryType


admin.site.register(EntryType, EntryTypeAdmin)


class UserActionAdmin(admin.ModelAdmin):
    model = UserAction
    fields = [
        "object_id",
        "content_type",
        "who",
        "when",
        "what",
        "why",
    ]

    list_display = (
        "object_id",
        "content_type",
        "who",
        "when",
        "what",
        "why",
    )

    readonly_fields = ["when"]
    ordering = ["-when"]


admin.site.register(UserAction, UserActionAdmin)


class CommunicationsLogEntryAdmin(admin.ModelAdmin):
    model = CommunicationsLogEntry
    list_display = (
        "object_id",
        "content_type",
        "to",
        "fromm",
        "entry_type",
    )
    ordering = ["-created"]


admin.site.register(CommunicationsLogEntry, CommunicationsLogEntryAdmin)



import logging

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from rest_framework.viewsets import ModelViewSet

from org_model_logs.models import UserAction
from org_model_logs.serializers import UserActionSerializer

logger = logging.getLogger(__name__)


class BaseUserActionViewSet(ModelViewSet):
    def log_user_action(self, object_id, action, why=None):
        model = self.model
        content_type = ContentType.objects.get_for_model(model)
        user_action = UserAction.objects.log_action(
            object_id=object_id,
            content_type=content_type,
            who=self.request.user.id,
            what=action.format(model._meta.model.__name__, object_id),
            why=why,
        )
        return user_action

    def create(self, request, *args, **kwargs):
        why = self.request.data.get("why", None)
        response = super().create(request, *args, **kwargs)
        instance = response.data
        user_action = self.log_user_action(
            instance["id"], settings.ACTION_CREATE, why=why
        )
        response.data["user_action"] = UserActionSerializer(user_action).data
        return response

    def retrieve(self, request, *args, **kwargs):
        response = super().retrieve(request, *args, **kwargs)
        instance = response.data
        self.log_user_action(instance["id"], settings.ACTION_VIEW)
        return response

    def update(self, request, *args, **kwargs):
        why = self.request.data.get("why", None)
        response = super().update(request, *args, **kwargs)
        instance = response.data
        user_action = self.log_user_action(
            instance["id"], settings.ACTION_UPDATE, why=why
        )
        response.data["user_action"] = UserActionSerializer(user_action).data
        return response

    def partial_update(self, request, *args, **kwargs):
        why = self.request.data.get("why", None)
        response = super().partial_update(request, *args, **kwargs)
        instance = response.data
        user_action = self.log_user_action(
            instance["id"], settings.ACTION_PARTIAL_UPDATE, why=why
        )
        response.data["user_action"] = UserActionSerializer(user_action).data
        return response

    def destroy(self, request, *args, **kwargs):
        why = self.request.data.get("why", None)
        response = super().destroy(request, *args, **kwargs)
        instance = response.data
        user_action = self.log_user_action(
            instance["id"], settings.ACTION_DESTROY, why=why
        )
        response.data["user_action"] = UserActionSerializer(user_action).data
        return response



from django.conf.urls import url
from rest_framework import routers

from org_model_logs.api import UserActionList

router = routers.SimpleRouter()

urlpatterns = [
    url(r"user-actions", UserActionList.as_view()),
]



# Generated by Django 3.2.13 on 2022-08-04 07:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0003_alter_useraction_unique_together'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='communicationslogentry',
            unique_together=set(),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-21 08:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.CharField(help_text='Primary key of the model.', max_length=191)),
                ('who', models.IntegerField()),
                ('when', models.DateTimeField(auto_now_add=True)),
                ('what', models.TextField()),
                ('content_type', models.ForeignKey(help_text='Content type of the model.', on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
        ),
        migrations.CreateModel(
            name='CommunicationsLogEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.CharField(help_text='Primary key of the model.', max_length=191)),
                ('to', models.TextField(blank=True, verbose_name='To')),
                ('fromm', models.CharField(blank=True, max_length=200, verbose_name='From')),
                ('cc', models.TextField(blank=True, verbose_name='cc')),
                ('type', models.CharField(choices=[('email', 'Email'), ('phone', 'Phone Call'), ('mail', 'Mail'), ('person', 'In Person'), ('onhold', 'On Hold'), ('onhold_remove', 'Remove On Hold'), ('with_qaofficer', 'With QA Officer'), ('with_qaofficer_completed', 'QA Officer Completed'), ('referral_complete', 'Referral Completed')], default='email', max_length=35)),
                ('reference', models.CharField(blank=True, max_length=100)),
                ('subject', models.CharField(blank=True, max_length=200, verbose_name='Subject / Description')),
                ('text', models.TextField(blank=True)),
                ('customer', models.IntegerField()),
                ('staff', models.IntegerField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(help_text='Content type of the model.', on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
        ),
        migrations.AddIndex(
            model_name='useraction',
            index=models.Index(fields=['content_type', 'object_id'], name='org_model_l_content_a3d2eb_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='useraction',
            unique_together={('content_type', 'object_id')},
        ),
        migrations.AddIndex(
            model_name='communicationslogentry',
            index=models.Index(fields=['content_type', 'object_id'], name='org_model_l_content_4eadf2_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='communicationslogentry',
            unique_together={('content_type', 'object_id')},
        ),
    ]



from django.apps import AppConfig


class OrgModelLogsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "org_model_logs"
    verbose_name = "Park Pass Model Logs"



# Generated by Django 3.2.13 on 2022-08-12 05:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0004_alter_communicationslogentry_unique_together'),
    ]

    operations = [
        migrations.AddField(
            model_name='useraction',
            name='why',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-02 05:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0005_useraction_why'),
    ]

    operations = [
        migrations.CreateModel(
            name='EntryType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Entry Type',
                'verbose_name_plural': 'Entry Types',
            },
        ),
        migrations.RemoveField(
            model_name='communicationslogentry',
            name='type',
        ),
        migrations.AddField(
            model_name='communicationslogentry',
            name='entry_type',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.PROTECT, to='org_model_logs.entrytype'),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['org_model_logs', 'org_model_logs.migrations']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'django-org-model-logs',
    'version': '1.1.0',
    'description': 'A Django app to log actions and communications about models.',
    'long_description': None,
    'author': 'DBCA OIM',
    'author_email': 'asi@dbca.wa.gov.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
}


setup(**setup_kwargs)



"""
    This module allows you to easily add User Actions and Communications
    Log Entries to any model in a project.
"""
from django.contrib.contenttypes.models import ContentType
from django.db import models


class UserActionManager(models.Manager):
    """This manager adds convenience methods for querying User Actions"""

    def log_action(self, content_type, object_id, who, what, why=None):
        return self.model.objects.create(
            content_type=content_type,
            object_id=str(object_id),
            who=who,
            what=what,
            why=why,
        )

    def get_for_model(self, model):
        content_type = ContentType.objects.get(model._meta.model)
        return self.filter(content_type=content_type)

    def get_for_object_reference(self, model, object_id):
        return self.get_for_model(model).filter(object_id=object_id)

    def get_for_object(self, obj, model_db=None):
        return self.get_for_object_reference(obj.__class__, obj.pk)


class UserAction(models.Model):
    """A class to represent a log record of who did what and when.

    User actions can be attached to any model in a project
    """

    objects = UserActionManager()

    object_id = models.CharField(
        max_length=191,
        help_text="Primary key of the model.",
    )

    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        help_text="Content type of the model.",
    )

    who = models.IntegerField()  # EmailUserRO
    when = models.DateTimeField(null=False, blank=False, auto_now_add=True)
    what = models.TextField(null=False, blank=False)
    why = models.TextField(null=True, blank=True)

    def __str__(self):
        return "{what} ({who} at {when}) {why}".format(
            what=self.what, who=self.who, when=self.when, why=str(self.why or "")
        )

    class Meta:
        indexes = (models.Index(fields=["content_type", "object_id"]),)


class EntryType(models.Model):
    entry_type = models.CharField(max_length=100, null=False, blank=False)

    class Meta:
        verbose_name = "Entry Type"
        verbose_name_plural = "Entry Types"
        ordering = ["id"]

    def __str__(self):
        return self.entry_type


class CommunicationsLogEntryManager(models.Manager):
    """This manager adds convenience methods for querying Communication Event Logs."""

    def log_communication(
        self,
        content_type,
        object_id,
        to,
        fromm,
        entry_type,
        subject,
        text,
        customer=None,
        staff=None,
        cc="",
        reference="",
    ):
        return self.model.objects.create(
            content_type=content_type,
            object_id=str(object_id),
            to=to,
            fromm=fromm,
            cc=cc,
            entry_type=entry_type,
            reference=reference,
            subject=subject,
            text=text,
            customer=customer,
            staff=staff,
        )

    def get_for_model(self, model):
        content_type = ContentType.objects.get(model._meta.model)
        return self.filter(content_type=content_type)

    def get_for_object_reference(self, model, object_id):
        return self.get_for_model(model).filter(object_id=object_id)

    def get_for_object(self, obj, model_db=None):
        return self.get_for_object_reference(obj.__class__, obj.pk)

    def get_queryset(self):
        return super().get_queryset().select_related("entry_type")


class CommunicationsLogEntry(models.Model):
    """A class to represent communication log entry.

    User actions can be attached to any model in your project
    """

    objects = CommunicationsLogEntryManager()

    object_id = models.CharField(
        max_length=191,
        help_text="Primary key of the model.",
    )

    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        help_text="Content type of the model.",
    )

    to = models.TextField(blank=True, verbose_name="To")
    fromm = models.CharField(max_length=200, blank=True, verbose_name="From")
    cc = models.TextField(blank=True, verbose_name="cc")

    entry_type = models.ForeignKey(EntryType, on_delete=models.PROTECT, default=None)

    reference = models.CharField(max_length=100, blank=True)
    subject = models.CharField(
        max_length=200, blank=True, verbose_name="Subject / Description"
    )
    text = models.TextField(blank=True)
    # Allowing null for cases such as sending emails to voucher recipients who are not in our customer database
    customer = models.IntegerField(null=True)  # EmailUserRO
    staff = models.IntegerField()  # EmailUserRO

    created = models.DateTimeField(auto_now_add=True, null=False, blank=False)

    class Meta:
        indexes = (models.Index(fields=["content_type", "object_id"]),)
        verbose_name = "Communications Log Entry"
        verbose_name_plural = "Communications Log Entries"
        ordering = ["-created"]



# Generated by Django 3.2.13 on 2022-08-04 06:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0002_alter_communicationslogentry_options'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='useraction',
            unique_together=set(),
        ),
    ]



import logging

from django.contrib.contenttypes.models import ContentType
from rest_framework import generics, viewsets
from rest_framework.serializers import ValidationError
from rest_framework_datatables.filters import DatatablesFilterBackend
from rest_framework_datatables.pagination import DatatablesPageNumberPagination

from org_model_logs.models import CommunicationsLogEntry, EntryType, UserAction
from org_model_logs.serializers import (
    CommunicationsLogEntrySerializer,
    CreateCommunicationsLogEntrySerializer,
    EntryTypeSerializer,
    UserActionSerializer,
)
from org_model_logs.utils import BaseUserActionViewSet

logger = logging.getLogger(__name__)


class EntryTypeList(generics.ListAPIView):
    model = EntryType
    serializer_class = EntryTypeSerializer
    queryset = EntryType.objects.all()


class UserActionList(generics.ListAPIView):
    search_fields = ["what", "why"]
    model = UserAction
    serializer_class = UserActionSerializer
    pagination_class = DatatablesPageNumberPagination
    filter_backends = (DatatablesFilterBackend,)
    ordering = ("-when",)

    def get_queryset(self):
        app_label = self.request.query_params.get("app_label", None)
        model = self.request.query_params.get("model", None)
        object_id = self.request.query_params.get("object_id", None)
        if app_label and model and object_id:
            if ContentType.objects.filter(app_label=app_label, model=model).exists():
                content_type = ContentType.objects.get(app_label=app_label, model=model)
                logger.debug("content_type = " + str(content_type))
                return UserAction.objects.filter(
                    content_type=content_type, object_id=object_id
                )
        if app_label and model:
            if ContentType.objects.filter(app_label=app_label, model=model).exists():
                content_type = ContentType.objects.get(app_label=app_label, model=model)
                return UserAction.objects.filter(content_type=content_type)
        if app_label:
            if ContentType.objects.filter(app_label=app_label).exists():
                content_types = ContentType.objects.filter(app_label=app_label)
                return UserAction.objects.filter(content_type__in=content_types)
        return UserAction.objects.all()

    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class UserActionViewSet(BaseUserActionViewSet):
    model = UserAction
    serializer_class = UserActionSerializer


class CreateCommunicationsLogEntry(generics.ListCreateAPIView):
    model = CommunicationsLogEntry
    filter_backends = (DatatablesFilterBackend,)
    pagination_class = DatatablesPageNumberPagination

    def get_serializer_class(self):
        if "POST" == self.request.method:
            return CreateCommunicationsLogEntrySerializer
        return CommunicationsLogEntrySerializer

    def get_queryset(self):
        app_label = self.request.query_params.get("app_label", None)
        model = self.request.query_params.get("model", None)
        object_id = self.request.query_params.get("object_id", None)
        if app_label and model and object_id:
            if ContentType.objects.filter(app_label=app_label, model=model).exists():
                content_type = ContentType.objects.get(app_label=app_label, model=model)
                return CommunicationsLogEntry.objects.filter(
                    content_type=content_type, object_id=object_id
                )
        if app_label and model:
            if ContentType.objects.filter(app_label=app_label, model=model).exists():
                content_type = ContentType.objects.get(app_label=app_label, model=model)
                return CommunicationsLogEntry.objects.filter(content_type=content_type)
        if app_label:
            if ContentType.objects.filter(app_label=app_label).exists():
                content_types = ContentType.objects.filter(app_label=app_label)
                return CommunicationsLogEntry.objects.filter(
                    content_type__in=content_types
                )
        return CommunicationsLogEntry.objects.all()

    def perform_create(self, serializer):
        app_label = serializer.validated_data.pop("app_label", None)
        model = serializer.validated_data.pop("model", None)
        logger.debug(str(serializer.validated_data))
        if not ContentType.objects.filter(app_label=app_label, model=model).exists():
            raise ValidationError(
                f"There is no content_type with app_label={app_label} and model={model}"
            )
        content_type = ContentType.objects.get(app_label=app_label, model=model)
        serializer.save(content_type=content_type, staff=self.request.user.id)



# Generated by Django 3.2.13 on 2022-06-23 02:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('org_model_logs', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='communicationslogentry',
            options={'verbose_name': 'Communications Log Entry', 'verbose_name_plural': 'Communications Log Entries'},
        ),
    ]
